@@ -25,8 +25,6 @@ class Connection(object):
         while self.worker.enabled:
             try:
                 self._handle()
-            except KeyboardInterrupt:
-                break
             except:
                 logger.error('exc occur. worker: %s',
                              self.worker, exc_info=True)
@@ -66,9 +64,8 @@ class Connection(object):
     def _connect(self):
         try:
             self.client.connect()
-        except KeyboardInterrupt:
-            raise
         except:
+            logger.error('exc occur. worker: %s', self.worker, exc_info=True)
             return False
         else:
             self.worker.app.events.open_conn(self)
