@@ -16,7 +16,6 @@ class Request(object):
     # 封装的task，外面不需要理解
     task = None
     is_valid = False
-    blueprint = None
     route_rule = None
     # 是否中断处理，即不调用view_func，主要用在before_request中
     interrupted = False
@@ -46,18 +45,23 @@ class Request(object):
         if self.cmd is None:
             return
 
-        route_rule = self.app.get_route_rule(self.cmd)
-        if route_rule:
-            # 在app层，直接返回
-            self.route_rule = route_rule
-            return
+        self.route_rule = self.app.get_route_rule(self.cmd)
 
-        for bp in self.app.blueprints:
-            route_rule = bp.get_route_rule(self.cmd)
-            if route_rule:
-                self.blueprint = bp
-                self.route_rule = route_rule
-                break
+    @property
+    def cmd(self):
+        return self.box.cmd if self.box else None
+
+    @property
+    def view_func(self):
+        return self.route_rule['view_func'] if self.route_rule else None
+
+    @property
+    def endpoint(self):
+        return self.route_rule['endpoint'] if self.route_rule else None
+
+    @property
+    def blueprint(self):
+        return self.route_rule.get('blueprint') if self.route_rule else None
 
     @property
     def worker(self):
@@ -75,26 +79,6 @@ class Request(object):
         """
         return self.task.client_ip
 
-    @property
-    def cmd(self):
-        try:
-            return self.box.cmd
-        except:
-            return None
-
-    @property
-    def view_func(self):
-        return self.route_rule['view_func'] if self.route_rule else None
-
-    @property
-    def endpoint(self):
-        if not self.route_rule:
-            return None
-
-        bp_endpoint = self.route_rule['endpoint']
-
-        return '.'.join([self.blueprint.name, bp_endpoint] if self.blueprint else [bp_endpoint])
-
     def make_rsp(self, data):
         """
         生成要写回的数据。业务层代码不要调用
