@@ -24,9 +24,6 @@ class Worker(object):
 
     got_first_request = False
 
-    # 是否已经发送KeyboardInterrupt
-    _got_keyboard_interrupt = False
-
     def __init__(self, app, group_id):
         """
         构造函数
@@ -50,8 +47,6 @@ class Worker(object):
             )
             self.conn = self.connection_class(self, address, self.app.config['WORKER_CONN_TIMEOUT'])
             self.conn.run()
-        except KeyboardInterrupt:
-            pass
         except:
             logger.error('exc occur. worker: %s', self, exc_info=True)
         finally:
@@ -68,27 +63,16 @@ class Worker(object):
         self.app.events.stop_worker(self)
 
     def _handle_proc_signals(self):
-        def stop_handler(signum, frame):
-            safe_stop_handler(signum, frame)
-
-            # 防止重复发送KeyboardInterrupt
-            # 因为在终端直接CTRL-C是会自动往子进程发送的
-            # 但是kill -INT不会
-            if not self._got_keyboard_interrupt:
-                self._got_keyboard_interrupt = True
-                raise KeyboardInterrupt
-
         def safe_stop_handler(signum, frame):
             self.enabled = False
 
             # 关闭读，从而快速退出
             safe_call(lambda: self.conn.shutdown(0))
 
-        # 强制结束，抛出异常终止程序进行
-        signal.signal(signal.SIGINT, stop_handler)
         # 安全停止
         signal.signal(signal.SIGTERM, safe_stop_handler)
-        signal.signal(signal.SIGHUP, safe_stop_handler)
+        signal.signal(signal.SIGINT, signal.SIG_IGN)
+        signal.signal(signal.SIGHUP, signal.SIG_IGN)
 
     def __repr__(self):
         return '<%s name: %s, group_id: %r>' % (
